# -*- coding: utf-8 -*-
# vim:enc=utf8:fenc=utf8:sts=4:sw=4:sta:et:nofen:si:ai:nowrap
# (C) Djordjevic Nebojsa <nesh@studioquattro.co.yu> 2005
# You're welcome to redistribute this software under the
# terms of the GNU Library General Public Licence version 2.0
# or, at your option, any higher version.
#
# You can read the complete GNU LGPL in the file COPYING
# which should come along with this software, or visit
# the Free Software Foundation's WEB site http://www.fsf.org
#

""" global i18n functions for use in core and similar
    modules

    based on:
       mailman i18n module
       roundup i18n module
"""

##################################################
## DEPENDENCIES ##

import sys
import os
import gettext as gettext_module
from django.models.i18n import locales, translations
from django.core.cache import cache
##################################################
## GLOBALS AND CONSTANTS ##

CACHE_TIMEOUT = 60 * 60 * 60 # 1h

##################################################
## CLASSES ##

class NullTranslations(gettext_module.NullTranslations):
    def __init__(self):
        gettext_module.NullTranslations.__init__(self)
    #

    def get_locale(self):
        return None
    # get_locale
# NullTranslation

# TODO: add support for site and package
class Translations(gettext_module.GNUTranslations):
    def __init__(self, locale_id):
        gettext_module.GNUTranslations.__init__(self)
        self._locale = None
        self._parse(locale_id)
    #
    def __repr__(self):
        if self._locale:
            return 'Translation for %s' % self._locale.locale
        else:
            return 'Translatin with no locale!!'
    # __repr__

    def get_locale(self):
        if self._locale is not None:
            return self._locale.locale
        else:
            return None
    # get_locale

    def _parse(self, locale_id):
        self._info = ['database generated']

        # try cache
        catalog = cache.get('CATALOG %d' % locale_id)
        locale = cache.get('LOCALE %d' % locale_id)

        if catalog is not None:
            self._catalog = catalog
        else:
            self._catalog = catalog = {}

        if locale is not None:
            self._locale = locale
        else:
            self._locale = locale = locales.get_object(pk=locale_id)
            # store locale
            cache.set('LOCALE %d' % locale_id, locale, CACHE_TIMEOUT)

        self._charset = locale.encoding

        if locale.get_plural() is not None:
            self.plural = gettext_module.c2py(locale.get_plural().plural)
        else:
            self.plural = lambda n: int(n != 1) # germanic plural by default
        #

        if catalog is None:
            # load all translations for all applications ....
            for trans in locale.get_translation_list():
                msg = trans.get_message()
                if trans.plural is not None:
                    # plural form
                    assert(msg.has_plural) # sanity check
                    msgid1 = unicode(msg.message, locale.encoding)
                    if msgid in catalog:
                        # we already added all translations for that msgid
                        continue
                    tmsg = [unicode(x, locale.encoding) for x in msg.get_translation_list(locale__exact=locale.id, order_by=['plural'])]
                    for i in range(len(tmsg)):
                        catalog[(msgid1, i)] = tmsg[i]
                else:
                    # standard msg
                    catalog[unicode(msg.message, locale.encoding)] = unicode(trans.text, locale.encoding)
            # for
            # store catalog
            cache.set('CATALOG %d' % locale_id, locale, CACHE_TIMEOUT)
        # if
    # _parse
# Translation

from django.core.extensions import DjangoContext
class I18NContext(DjangoContext):
    def __init__(self, request, dict=None):
        DjangoContext.__init__(self, request, dict)
        self.update(self.extra_context(request))
    # init

    @staticmethod
    def extra_context(request, data=None):
        """ return dict with extra i18n context data (if available) """
        if data is not None:
            ret = data
        else:
            ret = {}

        ret['locale'] = get_translation().get_locale()
        ret['avail_locales'] = available_locales()
        return ret
    #
# I18NContext

##################################################
## FUNCTIONS ##

##################################################
## locale info ##

def locale_choices():
    """ make choice list for available locales """
    lst = cache.get('LOCALE_CHOICES')
    if lst is not None:
        return lst

    try:
        lst = [(loc.locale, loc.name) for loc in locales.get_list()]
    except:
        return []
    cache.set('LOCALE_CHOICES', lst, CACHE_TIMEOUT)
    return lst
# locale_choices

def available_locales():
    lst = cache.get('LOCALE_LIST')

    if lst is None:
        lst = locales.get_list()
        cache.set('LOCALE_LIST', lst, CACHE_TIMEOUT)
    #

    return lst
# available_locales

# XXX cache this?
def _find_locales(language=None):
    # TODO: normalize locale names
    if language is None:
        return []
    elif isinstance(language, basestring):
        languages = [language]
    else:
        languages = language

    # get available locale id's
    lst = []
    for lang in languages:
        try:
            lid = locales.get_object(locale__exact=lang).id
        except locales.LocaleDoesNotExist:
            continue
        lst.append(lid)
    # for
    return lst
# _find_locales

def _get_translation(language=None):
    # get list of locales
    locales = _find_locales(language)

    # fallback to english unless english is selected language
    en = _find_locales('en')

    if en and (en[0] not in locales):
        locales.append(en[0])

    if locales:
        translator = Translations(locales[0])
        for locale in locales[1:]:
            # add fallback to the end of fallback chain
            translator.add_fallback(Translations(locale))
        #
    else:
        translator = NullTranslations()

    return translator
# _get_translation

########################################################
## main interface for setting and geting translations ##

import threading
_translations = {}

def get_translation(language=None):
    """ get current translation for the current thread
        translations are global for the current thread
    """
    global _translations
    thread = threading.currentThread()

    # translation already exists, ignore language
    if thread in _translations:
        return _translations[thread]
    #

    # no translaton, create new
    ret = _translations[thread] = _get_translation(language)
    return ret
# get_translation

def set_translation(language=None):
    """ set current *thread* global translation """
    global _translations
    thread = threading.currentThread()
    _translations[thread] = _get_translation(language)
# set_translation

from django.core.exceptions import ObjectDoesNotExist
# TODO: cache?
def get_for_lang(func, fallback='no translation', locale_field='locale', text_field='text', alt_loc=[]):
    """ get translation from db

        func: function to call to get object
        fallback: fallback text
        locale_field: locale id field
        text_field: text to return field
        alt_loc: list of alternative locales, 'en' is added if not present
    """

    locale = get_translation().get_locale()
    kwargs = {'%s__exact' % locale_field: locale}

    text = None
    cl = ''

    # try preffered locale
    try:
        obj = func(**kwargs)
        return getattr(obj, text_field)
    except ObjectDoesNotExist:
        pass

    if not isinstance(alt_loc, (tuple, list)):
        alt_loc = [alt_loc]
    #
    if 'en' not in alt_loc:
        alt_loc.append('en')
    #

    for loc in alt_loc:
        try:
            kwargs = {'%s__exact' % locale_field: loc}
            obj = func(**kwargs)
            text = getattr(obj, text_field)
            cl = loc
            break
        except ObjectDoesNotExist:
            pass
    # for

    if text is None:
        return '[%(locale)s] %(fallback)s' % locals()
    else:
        return '[%(cl)s] %(text)s' % locals()
# get_for_lang

##################################################
## parsing ##

def locales_from_request(req):
    """ find preffered locales from Accept-Language header, session or cookie
    """

    # try session
    try:
        if req.session.get('lang', False):
            return req.session.get('lang')
    except AttributeError:
        pass

    if 'lang' in req.COOKIES:
        return req.COOKIES['lang']

    # try to find out preffered language from the browser
    accept = req.META.get('HTTP_ACCEPT_LANGUAGE', None)
    if accept is not None:
        locales = _parse_accept(accept)
        if locales:
            return locales
    #

    # try to get default locale from config
    try:
        from django.conf.settings import DEFAULT_LOCALE
        return DEFAULT_LOCALE
    except ImportError:
        return ['en'] # if all else fails return 'en'
    #
# locales_from_request

def _parse_accept(accept):
    """ parse accept-language header """

    lst = accept.split(',')
    langs = {}

    for l in lst:
        if ';' in l:
            # have preference order
            lang, order = l.split(';')
            langs[int(float(order.strip()[2:]) * 100)] = lang.strip()[:2]
        else:
            langs[100] = l.strip()[:2]
    # for

    # sort locales by user prefferences
    order = langs.keys()
    order.sort()

    locales = []
    for locale in order:
        if langs[locale] not in locales:
            locales.append(langs[locale])
    #

    return locales
# parse_accept