# -*- coding: utf-8 -*-
# (C) Djordjevic Nebojsa <nesh@studioquattro.co.yu> 2005
# You're welcome to redistribute this software under the
# terms of the GNU Library General Public Licence version 2.0
# or, at your option, any higher version.
#
# You can read the complete GNU LGPL in the file COPYING
# which should come along with this software, or visit
# the Free Software Foundation's WEB site http://www.fsf.org
#

""" i18n support modeles """

##################################################
## DEPENDENCIES ##

from django.core import meta
from django.models.core import sites, packages
from encodings.aliases import aliases
##################################################
## GLOBALS ##

# make list of encodings
def _tmp():
    """ temp function to make list of encodings """
    al = aliases.values()
    lst = []
    for l in al:
        name = l.replace('_', ' ')
        tpl = (l, name)
        if tpl not in lst:
            lst.append(tpl)
    lst.sort()
    return lst
#
ENCODINGS = _tmp()
del _tmp

# make default SITE_ID
import django.conf.settings
try:
    SITE_ID = sites.get_object(pk=django.conf.settings.SITE_ID)
except sites.SiteDoesNotExists:
    SITE_ID=None
#

##################################################
## CLASSES ##

class Plural(meta.Model):
    """ plural form expressions """
    name = meta.CharField(maxlength=200, unique=True)
    description = meta.TextField(blank=True, null=True,
                                 help_text='use markdown tags or raw HTML')
    nplurals = meta.PositiveSmallIntegerField('number of plural forms')
    plural = meta.TextField('expression',
                            help_text='plural form expression, see: <a href="http://www.gnu.org/software/gettext/manual/html_node/gettext_150.html" target="_blank">gettext plural doc</a>',
                           )

    def __repr__(self):
        return self.name
    #

    def _manipulator_validate_plural(self, field_data, all_data):
        """ validate plurals """
        field_data = field_data.strip()
        if field_data == '':
            return ''

        if all_data['nplurals'].strip() == '':
            from django.core import validators
            raise validators.ValidationError('No nplurals is defined!')

        # try to compile plural expression
        try:
            from gettext import c2py
            c2py(field_data)
        except ValueError, err:
            from django.core import validators
            raise validators.ValidationError('Plural form expression error: %s' % err)
    # _manipulator_validate_plural

    class META:
        ordering = ['name']
        verbose_name = 'plural expresion'
        admin = meta.Admin(
                           list_display = ('name', 'nplurals', 'plural', 'description',),
                           fields = (
                                      (None, { 'fields': ('name', 'description',),}),
                                      ('Plurals', { 'fields': ('nplurals', 'plural'),}),
                                      ),
                           )
    # META
# Plural

class Locale(meta.Model):
    """ available languages """
    locale = meta.CharField(maxlength=5, unique=True)
    encoding = meta.CharField(maxlength=50, choices=ENCODINGS, default='utf_8')
    name = meta.CharField(maxlength=200, unique=True)
    plural = meta.ForeignKey(Plural, blank=True, null=True)

    def __repr__(self):
        return self.name
    #

    class META:
        ordering = ['locale']
        admin = meta.Admin(
            fields = (
                      (None, { 'fields': (('locale', 'encoding'), 'name', 'plural'),}),
                      ),
            list_display = ('name', 'locale', 'encoding', 'plural'),
            list_filter = ('plural', 'encoding'),
        )
    # META
# Locale

class Message(meta.Model):
    """ messages """
    site = meta.ForeignKey(sites.Site, blank=True, null=True, default=SITE_ID)
    package = meta.ForeignKey(packages.Package, blank=True, null=True)

    message = meta.CharField(maxlength=255)
    has_plural = meta.BooleanField(default=False)

    def __repr__(self):
        return self.message
    #

    def get_translations(self):
        lst = []
        for t in self.get_translation_list():
            lst.append(t.get_locale().name)
        return ', '.join(lst)
    #
    get_translations.short_description = 'translated for'

    class META:
        unique_together = (('site', 'package', 'message'),)
        admin = meta.Admin(
            fields = (
                      ('For', { 'fields': (('site', 'package'),),}),
                      (None, { 'fields': (('message', 'has_plural'),)}),
                      ),
            list_display = ('message', 'site', 'package', 'has_plural', 'get_translations'),
            list_filter = ('site', 'package', 'has_plural',), #('site', 'package'),
           )
    # META
# Message

class Translation(meta.Model):
    """ translation of the message """
    locale = meta.ForeignKey(Locale)
    message = meta.ForeignKey(Message, edit_inline=meta.TABULAR, num_in_admin=2, num_extra_on_change=1)
    plural = meta.PositiveIntegerField(blank=True, null=True, default=None)
    text = meta.TextField(core=True)

    def _manipulator_validate_plural(self, field_data, all_data):
        """ validate plurals """
        field_data = field_data.strip()
        if field_data == '':
            return ''

        pl = int(field_data)
        locale = self.get_locale()
        message = self.get_message()

        if locale.plural is None:
            from django.core import validators
            raise validators.ValidationError('No plural form is defined for this locale!')

        nplurals = locale.get_plural().nplurals
        if pl >= nplurals:
            from django.core import validators
            raise validators.ValidationError('Plural form must be less than %s' % nplurals - 1)

        if not message.has_plural:
            from django.core import validators
            raise validators.ValidationError("Message for this translation dosen't have has_plural set")
    # _manipulator_validate_plural

    def __repr__(self):
        if self.plural:
            return '%s[%d]: %s' % (self.get_locale().locale, self.plural, self.text)
        else:
            return '%s: %s' % (self.get_locale().locale, self.text)
    #

    #class META:
    #    unique_together = (('locale', 'message', 'plural'),)
    # META
# Translation